@@ -421,7 +421,6 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 					if not bmp.IsOk():
 						path = None
 			if not path:
-				from log import safe_print
 				safe_print("Warning: Missing bitmap '%s'" % name)
 				img = wx.EmptyImage(w, h)
 				img.SetMaskColour(0, 0, 0)
@@ -496,7 +495,6 @@ def get_argyll_display_number(geometry):
 	for i, display in enumerate(getcfg("displays")):
 		if display.find("@ " + geometry) > -1:
 			if debug:
-				from log import safe_print
 				safe_print("[D] Found display %s at index %i" % 
 						   (geometry, i))
 			return i
@@ -518,7 +516,6 @@ def get_display_number(display_no):
 			geometry = "%i, %i, %ix%i" % tuple(wx.Display(i).Geometry)
 			if display.endswith("@ " + geometry):
 				if debug:
-					from log import safe_print
 					safe_print("[D] Found display %s at index %i" % 
 							   (geometry, i))
 				return i
@@ -622,7 +619,6 @@ def get_data_path(relpath, rex=None):
 				try:
 					filelist = listdir_re(curpath, rex)
 				except Exception, exception:
-					from log import safe_print
 					safe_print(u"Error - directory '%s' listing failed: %s" % 
 							   tuple(safe_unicode(s) for s in (curpath, exception)))
 				else:
@@ -652,10 +648,9 @@ def runtimeconfig(pyfile):
 	attribute).
 	
 	"""
-	from log import setup_logging
+	global safe_print
+	from log import setup_logging, safe_print
 	setup_logging(logdir, pyname, pyext, confighome=confighome)
-	if debug or verbose >= 1:
-		from log import safe_print
 	if debug:
 		safe_print("[D] pydir:", pydir)
 	if isapp:
@@ -878,6 +873,7 @@ defaults = {
 	"3dlut.rendering_intent": "aw",
 	"3dlut.use_abstract_profile": 0,
 	"3dlut.size": 65,
+	"3dlut.size.backup": 65,
 	"3dlut.tab.enable": 0,
 	"3dlut.tab.enable.backup": 0,
 	"3dlut.whitepoint.x": 0.3127,
@@ -949,16 +945,30 @@ defaults = {
 	"extra_args.dispread": "",
 	"extra_args.spotread": "",
 	"extra_args.targen": "",
+	"gamap_default_intent": "p",
+	"gamap_out_viewcond": None,
 	"gamap_profile": "",
 	"gamap_perceptual": 0,
 	"gamap_perceptual_intent": "p",
 	"gamap_saturation": 0,
 	"gamap_saturation_intent": "s",
-	"gamap_default_intent": "p",
+	"gamap_src_viewcond": None,
 	"gamma": 2.2,
 	"iccgamut.surface_detail": 10.0,
 	"instruments": "",
-	"last_launch": "99",
+	"last_3dlut_path": u"",
+	"last_archive_save_path": u"",
+	"last_cal_path": u"",
+	"last_cal_or_icc_path": u"",
+	"last_colorimeter_ti3_path": u"",
+	"last_testchart_export_path": u"",
+	"last_filedialog_path": u"",
+	"last_icc_path": u"",
+	"last_launch": "99",  # Version
+	"last_reference_ti3_path": u"",
+	"last_ti1_path": u"",
+	"last_ti3_path": u"",
+	"last_vrml_path": u"",
 	"log.autoshow": 0,
 	"log.show": 0,
 	"lang": "en",
@@ -1018,6 +1028,7 @@ defaults = {
 	"patterngenerator.quantize_bits": 0,
 	"patterngenerator.resolve": "CM",
 	"patterngenerator.resolve.port": 20002,
+	"patterngenerator.use_pattern_window": 0,
 	"patterngenerator.use_video_levels": 0,
 	"position.x": 50,
 	"position.y": 50,
@@ -1104,6 +1115,8 @@ defaults = {
 										   "spyder5elite.exe",
 										   "spyder5express.exe",
 										   "spyder5pro.exe",
+										   "spyderxelite.exe",
+										   "spyderxpro.exe",
 										   "dispcal.exe",
 										   "dispread.exe",
 										   "dispwin.exe",
@@ -1113,6 +1126,7 @@ defaults = {
 	"profile_loader.quantize_bits": 16,
 	"profile_loader.reset_gamma_ramps": 0,
 	"profile_loader.show_notifications": 0,
+	"profile_loader.smooth_bits": "8",
 	"profile_loader.track_other_processes": 1,
 	"profile_loader.tray_icon_animation_quality": 2,
 	"profile_loader.use_madhcnet": 0,
@@ -1138,6 +1152,8 @@ defaults = {
 	"size.scripting.h": 384,
 	"size.synthiccframe.w": 512,
 	"size.synthiccframe.h": 384,
+	"size.tcgen.w": 0,
+	"size.tcgen.h": 0,
 	"skip_legacy_serial_ports": 1,
 	"skip_scripts": 1,
 	"splash.zoom": 0,
@@ -1427,7 +1443,6 @@ standard_profiles = []
 def get_standard_profiles(paths_only=False):
 	if not standard_profiles:
 		import ICCProfile as ICCP
-		from log import safe_print
 		# Reference profiles (Argyll + DisplayCAL)
 		ref_icc = get_data_path("ref", "\.ic[cm]$") or []
 		# Other profiles installed on the system
@@ -1598,7 +1613,6 @@ def makecfgdir(which="user", worker=None):
 			try:
 				os.makedirs(confighome)
 			except Exception, exception:
-				from log import safe_print
 				safe_print(u"Warning - could not create configuration directory "
 						   "'%s': %s" % (confighome, safe_unicode(exception)))
 				return False
@@ -1617,7 +1631,6 @@ def makecfgdir(which="user", worker=None):
 				if isinstance(result, Exception):
 					raise result
 		except Exception, exception:
-			from log import safe_print
 			safe_print(u"Warning - could not create configuration directory "
 					   "'%s': %s" % (config_sys, safe_unicode(exception)))
 			return False
@@ -1668,9 +1681,17 @@ def initcfg(module=None, cfg=cfg, force_load=False):
 					safe_print(u"Warning - os.stat('%s') failed: %s" % 
 							   tuple(safe_unicode(s) for s in (cfgfile,
 															   exception)))
-				if force_load or mtime != cfginited.get(cfgfile):
+				last_checked = cfginited.get(cfgfile)
+				if force_load or mtime != last_checked:
 					cfginited[cfgfile] = mtime
 					cfgfiles.append(cfgfile)
+					if force_load:
+						msg = "Force loading"
+					elif last_checked:
+						msg = "Reloading"
+					else:
+						msg = "Loading"
+					safe_print(msg, cfgfile)
 				# Make user config take precedence
 				break
 	if not cfgfiles:
@@ -1683,7 +1704,6 @@ def initcfg(module=None, cfg=cfg, force_load=False):
 		# This won't raise an exception if the file does not exist, only
 		# if it can't be parsed
 	except Exception, exception:
-		from log import safe_print
 		safe_print("Warning - could not parse configuration files:\n%s" %
 				   "\n".join(cfgfiles))
 		# Fix Python 2.7 ConfigParser option values being lists instead of
@@ -1863,6 +1883,33 @@ def setcfg(name, value, cfg=cfg):
 		cfg.set(ConfigParser.DEFAULTSECT, name, unicode(value).encode("UTF-8"))
 
 
+def setcfg_cond(condition, name, value, set_if_backup_exists=False):
+	"""
+	If <condition>, backup configuration option <name> if not yet backed up
+	and set option to <value> if backup did not previously exist or
+	set_if_backup_exists evaluates to True
+	
+	If not <condition> and backed up option <name>, restore option <name> to
+	backed up value and discard backup
+	
+	Return whether or not configuration was changed
+
+	"""
+	changed = False
+	backup = getcfg(name + ".backup", False)
+	if condition:
+		if backup is None:
+			setcfg(name + ".backup", getcfg(name))
+		if backup is None or set_if_backup_exists:
+			setcfg(name, value)
+			changed = True
+	elif backup is not None:
+		setcfg(name, getcfg(name + ".backup"))
+		setcfg(name + ".backup", None)
+		changed = True
+	return changed
+
+
 def writecfg(which="user", worker=None, module=None, options=(), cfg=cfg):
 	"""
 	Write configuration file.
@@ -1875,6 +1922,11 @@ def writecfg(which="user", worker=None, module=None, options=(), cfg=cfg):
 		cfgbasename = "%s-%s" % (appbasename, module)
 	else:
 		cfgbasename = appbasename
+	# Remove unknown options
+	for name, val in cfg.items(ConfigParser.DEFAULTSECT):
+		if not name in defaults:
+			safe_print("Removing unknown option:", name)
+			setcfg(name, None)
 	if which == "user":
 		# user config - stores everything and overrides system-wide config
 		cfgfilename = os.path.join(confighome, cfgbasename + ".ini")
@@ -1897,7 +1949,6 @@ def writecfg(which="user", worker=None, module=None, options=(), cfg=cfg):
 			cfgfile.write(os.linesep.join(lines) + os.linesep)
 			cfgfile.close()
 		except Exception, exception:
-			from log import safe_print
 			safe_print(u"Warning - could not write user configuration file "
 					   "'%s': %s" % (cfgfilename, safe_unicode(exception)))
 			return False
@@ -1930,7 +1981,6 @@ def writecfg(which="user", worker=None, module=None, options=(), cfg=cfg):
 				if isinstance(result, Exception):
 					raise result
 		except Exception, exception:
-			from log import safe_print
 			safe_print(u"Warning - could not write system-wide configuration file "
 					   "'%s': %s" % (cfgfilename2, safe_unicode(exception)))
 			return False
